/************************************************************************************************

 Author        : ADI - Apps            www.analog.com/AduC7060

 Date          : November 2007

 File          : I2C_Master.c

 Hardware      : ADuC706x

 Description   : This I2C Master example shows how initialise the I2C peripheral in Master mode
 			This simple example transmits 6 bytes to Slave address 0xA0 after a reset.
			When an XIRQ0 interrupt occurs, the Master will then read back 6 bytes from the Slave.		
*************************************************************************************************/
// Bit Definitions
#define BIT0  0x01
#define BIT1  0x02
#define BIT2  0x04
#define BIT3  0x08
#define BIT4  0x10
#define BIT5  0x20
#define BIT6  0x40
#define BIT7  0x80
#define BIT8  0x100
#define BIT9  0x200
#define BIT10 0x400
#define BIT11 0x800
#define BIT12 0x1000
#define BIT13 0x2000
#define BIT14 0x4000
#define BIT15 0x8000
#define I2C_RD_BIT 0x1

#include<aduc7060.h>


unsigned char szTxData[] = {0x1, 0x2, 0x10, 0x20, 0x40, 0x80};	// Array to send to Slave
unsigned char ucTxCount = 0;  // Array index variable for szTxData[]
unsigned char szRxData[6];	  // Array for reading Data from Slave
unsigned char ucRxCount = 0;  // Array index variable for szRxData[]
unsigned char ucWaitingForXIRQ0 = 1; // Flag to begin reading from Slave - controlled by XIRQ0
int main(void)
{
    POWKEY1 = 0x1;
	POWCON0 = 0x78;		   // Set core to max CPU speed of 10.24Mhz
	POWKEY2 = 0xF4;
	// Configure P0.1 and P0.3 for I2C mode
	GP0CON0 = BIT4 + BIT12; // Select SPI/I2C alternative function for P0.1 & P0.3
	GP0KEY1 = 0x7;		   // Write to GP0KEY1
	GP0CON1	= BIT1;		   // Select I2C functionality for P0.1 & P0.3
	GP0KEY2 = 0x13;		   // Write to GP0KEY2

	// Enable I2C Master mode, baud rate and interrupt sources
	I2CMCON = BIT0 + BIT4  // Enable I2C Master + Enable Rx interrupt
			+ BIT5 + BIT8; // Enable Tx interrupt + Enable transmission complete interrupt
	I2CDIV  = 0x3131;	   // Select 100.3kHz clock rate
	IRQEN = BIT15; // + BIT13; // Enable I2C Master and XIRQ0 interrupts

	// Begin Master Transmit sequence
	ucTxCount = 0;
	I2CFSTA = BIT9;			// Flush Master Tx FIFO
	I2CFSTA &= ~BIT9;
	I2CMTX = szTxData[ucTxCount++];
	I2CADR0 = 0xA0; 
	ucWaitingForXIRQ0 = 1;
	ucRxCount = 0;
	while (1)
	{
	 	if (ucWaitingForXIRQ0 == 0)	// Wait for XIRQ0 to trigger - cleared in XIRQ0 ISR
		{
			ucWaitingForXIRQ0 = 1;
			ucRxCount = 0;
			I2CMCNT0 = 5; // Read back 6 bytes
			I2CADR0 =  0xA1;// + I2C_RD_BIT; // Begin Read sequence from Slave
		}
	}
   return 0x00;
}

void IRQ_Handler(void) __irq
{
	unsigned long IRQSTATUS = 0;
	unsigned int I2CMSTATUS = 0;

	IRQSTATUS = IRQSTA;	   // Read off IRQSTA register
	if ((IRQSTATUS & BIT13) == BIT13)	//XIRQ0 interrupt source
	{
		ucWaitingForXIRQ0 = 0;			// Enable Read request from Slave.
	}

	if ((IRQSTATUS & BIT15) == BIT15)	//If I2C Master interrupt source
	{
	   I2CMSTATUS = I2CMSTA;
	   if ((I2CMSTATUS & BIT2) == BIT2) // If I2C Master Tx IRQ
	   {
			if (ucTxCount < 6)		   // Have max 6 bytes been sent?
				I2CMTX = szTxData[ucTxCount++];	 // Load Tx buffer
	   }
	   if ((I2CMSTATUS & BIT3) == BIT3) // If I2C Master Rx IRQ
	   {
			if (ucRxCount < 6)		   // Have max 6 bytes been received?
			{
				szRxData[ucRxCount] = I2CMRX;  // Read Rx buffer
				ucRxCount++;
			}
	   }
	}
}


